//*****************************************************************************
// Copyright  2006, Xilinx, Inc. 
// This design is confidential and proprietary of Xilinx, Inc. All Rights Reserved.
//*****************************************************************************
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Version: 1.0
//  \   \        Filename: 
//  /   /        Date Last Modified: 
// /___/   /\    Date Created: 10-sep-06 
// \   \  /  \ 
//  \___\/\___\ 
// 
// Device: 
// Purpose:   
// Reference: 
//----------------------------------------------------------------------------
 
//   
//*****************************************************************************
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is." Xilinx and its licensors make and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, no infringement, or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrant or
//              make any representations regarding use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits, cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply notwithstanding the failure of the
//              essential purpose of any limited remedies herein.
//
//*****************************************************************************/

//----------------------------------------------------------------------------
// Filename:     bootload_graphics.c
// 
// Description:   Basic functions to plot on TFT display. 
//				  Low level driver in xtft_main.c
// Design Notes:
//   
//-----------------------------------------------------------------------------
// Xilinx APD
// History:
// 
//
//-----------------------------------------------------------------------------

#include "xio.h"
#include "xtft.h"
#include "xtft_l.h"

#include "sleep.h"
#include "math.h"
#include "bootload_basicgraphics.h"

#define DISPLAY_COLUMNS  640
#define DISPLAY_ROWS     480

#define NORM_FACTOR      1000000

/* values generated by touchcalibrate */
#define DEFAULT_XMULT    956
#define DEFAULT_YMULT    1192
#define DEFAULT_XLCORN   254057
#define DEFAULT_YLCORN   197499

/*   GLOBALS    */
int Xpixel, Ypixel, Xmult, Ymult, Xlcorn, Ylcorn;

#define TFT_WIDTH        640
#define CHAR_WIDTH       8
#define CHAR_HEIGHT      12

/* XTft_WriteString: Writes a string of characters to the TFT */
void XTft_WriteString(XTft *InstancePtr, unsigned char *val) 
{
  while (*val) {
    XTft_Write(InstancePtr, *val);
    val++;
  }
}

/* XTft_WriteColor: Writes a character with a specified colour */
void XTft_WriteColor(XTft *InstancePtr, unsigned char c, 
		     unsigned int fgCol, unsigned int bgCol) 
{
  XTft_SetColor(InstancePtr, fgCol, bgCol);
  XTft_Write(InstancePtr, (Xint8)c);
}

/* XTft_WriteColorString: Writes a string with a specified colour */
void XTft_WriteColorString(XTft *InstancePtr, 
			   unsigned char *val, unsigned int fgCol, unsigned int bgCol)
{
  while (*val) {
    XTft_WriteColor(InstancePtr, *val, fgCol, bgCol);
    val++;
  }
}

/* XTft_DrawLine: Draws a line between two points with
 * with a specified color                              */
int XTft_DrawLine(XTft *InstancePtr,
		  int x1, int y1, int x2, int y2, unsigned int col)
{
  
  int slope, yintercept;
  int xmin, ymin, xmax, ymax, i, j, mx;
  
  if (x1 >= 0 &&
      x1 <= DISPLAY_COLUMNS-1 &&
      x2 >= 0 &&
      x2 <= DISPLAY_COLUMNS-1 &&
      y1 >= 0 &&
      y1 <= DISPLAY_ROWS-1 &&
      y2 >= 0 &&
      y2 <= DISPLAY_ROWS-1) {
    
    if (x2-x1 != 0) {
      slope = ((y2-y1)/(x2-x1)*100000);     /* Calculate slope */
      yintercept = y1-((slope/100000)*x1);  /* Calculate yintercept */
    }
    else { /* divide by zero */
      slope = 0;
      yintercept = (-1);
    }
    
    if (x2 < x1) {
      xmin = x2;
      xmax = x1;
    }
    else {
      xmin = x1;
      xmax = x2;
    }
    if (y2 < y1) {
      ymin = y2;
      ymax = y1;
    }
    else {
      ymin = y1;
      ymax = y2;
    }

    for (i=xmin; i<=xmax; i++) {
      mx = (slope * i) / 100000;
      for (j=ymin; j<=ymax; j++) {
	if ((j - mx) == yintercept) { /* Calculate visible line */
	  XTft_SetPixel(InstancePtr, i, j, col);
	}
	else {
	  if ((slope == 0) && (yintercept == -1)) { /* divide by zero */
	    XTft_SetPixel(InstancePtr, i, j, col);  /* a.k.a vertical line */
	  }
	}
      }
    }
    return 0;
  }
  else {
    return 1;
  }
}

/* XTft_DrawEmptyBox:  Draws a box on the TFT of a specified
 * color between two points, with a given thickness          */
int XTft_DrawEmptyBox(XTft *InstancePtr,
		      int x1, int y1, int x2, int y2, unsigned int col, int thickness)
{
   int xmin, xmax, ymin, ymax, i;

   if (x1 >= 0 &&
       x1 <= DISPLAY_COLUMNS-1 &&
       x2 >= 0 &&
       x2 <= DISPLAY_COLUMNS-1 &&
       y1 >= 0 &&
       y1 <= DISPLAY_ROWS-1 &&
       y2 >= 0 &&
       y2 <= DISPLAY_ROWS-1) {
      if (x2 < x1) {
	xmin = x2;
	xmax = x1;
      }
      else {
	xmin = x1;
	xmax = x2;
      }
      if (y2 < y1) {
	ymin = y2;
	ymax = y1;
      }
      else {
	ymin = y1;
	ymax = y2;
      }

      for (i = 0;i < thickness;i++) {
        XTft_DrawLine(InstancePtr, xmin+i, ymin+i, xmax+i, ymin+i, col);
        XTft_DrawLine(InstancePtr, xmin+i, ymin+i, xmin+i, ymax+i, col);
        XTft_DrawLine(InstancePtr, xmax+i, ymax+i, xmax+i, ymin+i, col);
        XTft_DrawLine(InstancePtr, xmin+i, ymax+i, xmax+i, ymax+i, col);
      }
   }

   return 0;
}


/* XTft_DrawSolidBox:  Draws a box on the TFT of a specified
 * color between two points                                  */
int XTft_DrawSolidBox(XTft *InstancePtr, int x1, int y1, int x2, int y2, unsigned int col)
{
   int xmin,xmax,ymin,ymax,i,j;

   if (x1 >= 0 &&
       x1 <= DISPLAY_COLUMNS-1 &&
       x2 >= 0 &&
       x2 <= DISPLAY_COLUMNS-1 &&
       y1 >= 0 &&
       y1 <= DISPLAY_ROWS-1 &&
       y2 >= 0 &&
       y2 <= DISPLAY_ROWS-1) {
      if (x2 < x1) {
         xmin = x2;
         xmax = x1;
      }
      else {
         xmin = x1;
         xmax = x2;
      }
      if (y2 < y1) {
         ymin = y2;
         ymax = y1;
      }
      else {
         ymin = y1;
         ymax = y2;
      }

      for (i=xmin; i<=xmax; i++) {
         for (j=ymin; j<=ymax; j++) {
	   XTft_SetPixel(InstancePtr, i, j, col);
         }
      }
      return 0;
   }
   return 1;

}


//new
/* XTft_WriteColorString: Writes a string with a specified colour */
void XTft_BlinkingString(XTft *InstancePtr, 
			   unsigned char *val, unsigned int fgCol, unsigned int bgCol, int time, int x, int y)
{
  unsigned char *val_old, *space;
  int i,j;
  volatile float k = 1.1;
  val_old = val;
  space = " ";
  for(i=0;i<time;i++)
  {
		XTft_SetPos(InstancePtr, x, y);
		while (*val) {
			XTft_WriteColor(InstancePtr, *val, fgCol, bgCol);
			val++;
		}
		for(j=0;j<50000;j++)
		{
			k=1.37256*k;
		}
		val = val_old;
		XTft_SetPos(InstancePtr, x, y);
		while (*val) {
			XTft_WriteColor(InstancePtr, *space, fgCol, bgCol);
			val++;
		}
		for(j=0;j<50000;j++)
		{
			k=1.37256*k;
		}
		val = val_old;
	}
	
	val = val_old;
	XTft_SetPos(InstancePtr, x, y);
	while (*val) {
		XTft_WriteColor(InstancePtr, *val, fgCol, bgCol);
		val++;
	}
}

