/**

 * @file dct-test.c
 * DCT test. (c) 2001 Fabrice Bellard.
 * Started from sample code by Juan J. Sierralta P.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "co.h"
#include "dct_test.h"

#ifdef IMPULSE_C_TARGET
#include "xparameters.h"
#include "xcache_l.h"

#define printf xil_printf
extern void prep_idct(short *block);
extern void impc_idct(short *block, short *next);
#endif

#ifndef MAX
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))
#endif

#define ABS(x) (((x)>=0) ? (x) : -(x))

/* reference fdct/idct */
extern void Fast_IDCT(short *block);

static __attribute__ ((aligned (16))) short block[64];
static __attribute__ ((aligned (16))) short block1[64];


#ifdef IMPULSE_C_TARGET
void print16(char *msg, short *buf)

{
  int i;
  printf("%s",msg);
  for (i=0; i<8; i++) {
    printf(" %d",buf[i]);
  }
  printf("\r\n");
}
#else
void print16(char *msg, co_int32 *buf)

{
  int i;
  printf("%s",msg);
  for (i=0; i<4; i++) {
    printf(" %d %d",(int)(short)buf[i],buf[i]>>16);
  }
  printf("\r\n");
}
#endif

void dct_error()
{
    int it, i;
    int err_inf, v;
    co_int64 err2;
    co_int64 sysErr[64], sysErrMax=0;
    int maxout=0;
    int blockSumErrMax=0, blockSumErr;

    err_inf = 0;
    err2 = 0;
    for(i=0; i<64; i++) sysErr[i]=0;

	prep_idct(dct_test);
    for(it=0;it<TEST_BLOCKS;it++) {
//		printf("block %d:\r\n",it);
		impc_idct(block,dct_test+((it+1)*128));
#if 0
        for(i=0; i<64; i++)
            block[i]=dct_test[(it*128)+i];

        Fast_IDCT(block);
#endif

        for(i=0;i<64;i++)
            block1[i] = dct_test[(it*128)+64+i];

        blockSumErr=0;
        for(i=0;i<64;i++) {
            v = ABS(block[i] - block1[i]);
            if (v > err_inf)
                err_inf = v;
            err2 += v * v;
            sysErr[i] += block[i] - block1[i];
            blockSumErr += v;
            if( ABS(block[i])>maxout) maxout=ABS(block[i]);
        }
        if(blockSumErrMax < blockSumErr) blockSumErrMax= blockSumErr;
//		printf("\tblockSumErr=%d\r\n",blockSumErr);
    }
    for(i=0; i<64; i++) {
		sysErr[i]=ABS(sysErr[i]);
		sysErrMax= MAX(sysErrMax, sysErr[i]);
	}

    printf("IDCT: err_inf=%d err2=%d syserr=%d maxout=%d blockSumErr=%d\r\n",
           err_inf,(int)err2, (int)sysErrMax, maxout, blockSumErrMax);
	if (err_inf==1)
		printf("passed\r\n");
}

int main(int argc, char **argv)
{
#ifdef IMPULSE_C_TARGET
#ifdef WITH_CACHE
	XCache_EnableDCache(0x80000001);
	XCache_EnableICache(0x80000001);
#endif
#endif

    printf("begin\r\n");
	run_impc();
    printf("done\r\n");
	IF_SIM(getchar();)
    return 0;
}
