//*****************************************************************************
// Copyright  2006, Xilinx, Inc. 
// This design is confidential and proprietary of Xilinx, Inc. All Rights Reserved.
//*****************************************************************************
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Version: 1.0
//  \   \        Filename: 
//  /   /        Date Last Modified: 
// /___/   /\    Date Created: 10-sep-06 
// \   \  /  \ 
//  \___\/\___\ 
// 
// Device: 
// Purpose:   
// Reference: 
//----------------------------------------------------------------------------
 
//   
//*****************************************************************************
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is." Xilinx and its licensors make and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, no infringement, or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrant or
//              make any representations regarding use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits, cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply notwithstanding the failure of the
//              essential purpose of any limited remedies herein.
//
//*****************************************************************************/

//----------------------------------------------------------------------------
// Filename:     stopwatch.c
// 
// Description:    
// Stopwatch routines for code timing and comparison between software-only
//	and hardware-software implementations
// Design Notes:
//   
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Xilinx APD
// History:-----------------------------
//			07/2005 - Modified - KRS - Added timerNo parameter to have dual timers
//			08/2006 - Modified - GY - Added timerNo parameter to have three timers
//----------------------------------------------------------------------------

#define FLOAT_LIB

#include "stdio.h"
#include "xtime_l.h"
#include "xparameters.h"

// stop_watch routine
// supports split timming
// call with mode = 
//		0 reset timer (must always call first if using start)
//		1 reset and start timer
//		2 start/restart timer
//		3 pause/stop timer
//		4 stop timer and display total time
//		5 display total time
//		6 just return current time (without adding up to now)
// returns current total time in microseconds

int stop_watch(int mode, int timerNo)
{
// tStart is in system clock units
// tMark is in nanoseconds
 	XTime tCur;
	static XTime tStart0, tTotal0;
	static XTime tStart1, tTotal1;
static XTime tStart2, tTotal2;
	static XTime *tStart;
	static XTime *tTotal;
	int result;

	//pick one of the two timer variables based on timerNo

	switch (timerNo)
	{
		case 1:
			tStart = &tStart0;
			tTotal = &tTotal0;
			break;
		case 2:
			tStart = &tStart1;
			tTotal = &tTotal1;
			break;
		case 3:
			tStart = &tStart2;
			tTotal = &tTotal2;
			break;
		default:
			tStart = &tStart0;
			tTotal = &tTotal0;
			break;
	}
	
	// capture current time
	XTime_GetTime(&tCur);

	switch (mode)
	{
		case 0: // initialize timer
			*tTotal = 0;
			break;
		case 1: // initialize and start timer
			*tTotal = 0;
		case 2: // start or restart timer
			// save current time for stopwatch
			XTime_GetTime(tStart);
			break;
		case 3: // pause timer
			// compute total time up to now in processor cycles and add to previous value
			*tTotal += (tCur - *tStart);
			break;
		case 4: // stop timer and display time
			// compute total time up to now in processor cycles and add to previous value
			*tTotal += (tCur - *tStart);
			break;
		case 5: // display total time
			*tTotal += (tCur - *tStart);
			// compute total time in microseconds
			result = (*tTotal * (XTime) 1000000) / (XTime) XPAR_CPU_PPC440_CORE_CLOCK_FREQ_HZ;
			// output scaled answer
			if (result < 100000)
				xil_printf ("  Time (microsec): %d \n\r",result);
			else
			{
				result = (result +500) / 1000; // scale by 1000 with rounding
				if (result < 100000)
					xil_printf ("  Time (millisec): %d \n\r",result);
				else
				{
					result = (result +500) / 1000; // scale by 1000 with rounding
					xil_printf ("  Time (seconds): %d \n\r",result);
					}
				}
			break;
		case 6: // return current time (default after switch() )
			break;
		}
		result = (*tTotal * (XTime) 1000000) / (XTime) XPAR_CPU_PPC440_CORE_CLOCK_FREQ_HZ;
		return (result);
	}
