-------------------------------------------------------------------------------
-- dcr_intc_0_wrapper.vhd
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

library dcr_intc_v1_00_b;
use dcr_intc_v1_00_b.all;

entity dcr_intc_0_wrapper is
  port (
    DCR_Clk : in std_logic;
    Intr : in std_logic_vector(3 downto 0);
    DCR_Rst : in std_logic;
    DCR_Read : in std_logic;
    DCR_Write : in std_logic;
    DCR_ABus : in std_logic_vector(0 to 9);
    DCR_DBus : in std_logic_vector(0 to 31);
    Intc_dcrDBus : out std_logic_vector(0 to 31);
    Intc_dcrAck : out std_logic;
    Irq : out std_logic
  );

  attribute x_core_info : STRING;
  attribute x_core_info of dcr_intc_0_wrapper : entity is "dcr_intc_v1_00_b";

end dcr_intc_0_wrapper;

architecture STRUCTURE of dcr_intc_0_wrapper is

  component dcr_intc is
    generic (
      C_FAMILY : string;
      C_Y : integer;
      C_X : integer;
      C_U_SET : string;
      C_DCR_AWIDTH : integer;
      C_DCR_DWIDTH : integer;
      C_BASEADDR : std_logic_vector;
      C_HIGHADDR : std_logic_vector;
      C_NUM_INTR_INPUTS : integer;
      C_KIND_OF_INTR : std_logic_vector;
      C_KIND_OF_EDGE : std_logic_vector;
      C_KIND_OF_LVL : std_logic_vector;
      C_HAS_IPR : integer;
      C_HAS_SIE : integer;
      C_HAS_CIE : integer;
      C_HAS_IVR : integer;
      C_IRQ_IS_LEVEL : integer;
      C_IRQ_ACTIVE : std_logic
    );
    port (
      DCR_Clk : in std_logic;
      Intr : in std_logic_vector(C_NUM_INTR_INPUTS-1 downto 0);
      DCR_Rst : in std_logic;
      DCR_Read : in std_logic;
      DCR_Write : in std_logic;
      DCR_ABus : in std_logic_vector(0 to C_DCR_AWIDTH-1);
      DCR_DBus : in std_logic_vector(0 to C_DCR_DWIDTH-1);
      Intc_dcrDBus : out std_logic_vector(0 to C_DCR_DWIDTH-1);
      Intc_dcrAck : out std_logic;
      Irq : out std_logic
    );
  end component;

begin

  dcr_intc_0 : dcr_intc
    generic map (
      C_FAMILY => "virtex5",
      C_Y => 0,
      C_X => 0,
      C_U_SET => "intc",
      C_DCR_AWIDTH => 10,
      C_DCR_DWIDTH => 32,
      C_BASEADDR => B"1000000000",
      C_HIGHADDR => B"1011111111",
      C_NUM_INTR_INPUTS => 4,
      C_KIND_OF_INTR => B"00000000000000000000000000000000",
      C_KIND_OF_EDGE => B"00000000000000000000000000000000",
      C_KIND_OF_LVL => B"00000000000000000000000000001111",
      C_HAS_IPR => 1,
      C_HAS_SIE => 1,
      C_HAS_CIE => 1,
      C_HAS_IVR => 1,
      C_IRQ_IS_LEVEL => 1,
      C_IRQ_ACTIVE => '1'
    )
    port map (
      DCR_Clk => DCR_Clk,
      Intr => Intr,
      DCR_Rst => DCR_Rst,
      DCR_Read => DCR_Read,
      DCR_Write => DCR_Write,
      DCR_ABus => DCR_ABus,
      DCR_DBus => DCR_DBus,
      Intc_dcrDBus => Intc_dcrDBus,
      Intc_dcrAck => Intc_dcrAck,
      Irq => Irq
    );

end architecture STRUCTURE;

