-- 
-- Copyright (c) 2002-2007 by Impulse Accelerated Technologies, Inc.
-- All rights reserved.
-- 
-- This source file may be used and redistributed without charge
-- subject to the provisions of the IMPULSE ACCELERATED TECHNOLOGIES,
-- INC. REDISTRIBUTABLE IP LICENSE AGREEMENT, and provided that this
-- copyright statement is not removed from the file, and that any
-- derivative work contains this copyright notice.
-- 
library ieee;
use ieee.std_logic_1164.all;

use ieee.std_logic_arith.all;
library impulse;
use impulse.apu_if.all;

library apu_idct_colorconv_arch_v1_00_a;
use apu_idct_colorconv_arch_v1_00_a.all;


entity apu_idct_colorconv_arch is
  generic (
    C_FAMILY           : string                    := "virtex5fx"  );
  port (
    -- apu ports
    apufcminstruction : std_logic_vector(0 to 31);
    apufcminstrvalid : std_logic;
    apufcmradata : std_logic_vector(0 to 31);
    apufcmrbdata : std_logic_vector(0 to 31);
    apufcmoperandvalid : std_logic;
    apufcmflush : std_logic;
    apufcmwritebackok : std_logic;
    apufcmloaddata : std_logic_vector(0 to 127);
    apufcmloaddvalid : std_logic;
    apufcmloadbyteaddr : std_logic_vector(0 to 3);
    apufcmendian : std_logic;
    apufcmdecudi : std_logic_vector(0 to 3);
    apufcmdecudivalid : std_logic;
    apufcmdecfpuop : std_logic;
    apufcmdecload : std_logic;
    apufcmdecstore : std_logic;
    apufcmdecldstxfersize : std_logic_vector(0 to 2);
    apufcmdecnonauton : std_logic;
    apufcmnextinstrready : std_logic;
    fcmapuresult : inout std_logic_vector(0 to 31);
    fcmapudone : inout std_logic;
    fcmapusleepnotready : inout std_logic;
    fcmapuresultvalid : inout std_logic;
    fcmapucr : inout std_logic_vector(0 to 3);
    fcmapuexception : inout std_logic;
    fcmapustoredata : inout std_logic_vector(0 to 127);
    fcmapuconfirminstr : inout std_logic;
    fcmapufpscrfex : inout std_logic;
udidebug : out std_logic_vector(153 downto 0);
    apu_clk, reset : in std_logic
    );
end apu_idct_colorconv_arch;

architecture impl of apu_idct_colorconv_arch is
  component idct_colorconv_arch is
    port (
    reset : in std_ulogic;
    sclk : in std_ulogic;
    clk : in std_ulogic;
    p_Producer_istream_en : in std_ulogic;
    p_Producer_istream_eos : in std_ulogic;
    p_Producer_istream_data : in std_ulogic_vector (127 downto 0);
    p_Producer_istream_rdy : out std_ulogic;
    p_Consumer_ostream_en : in std_ulogic;
    p_Consumer_ostream_data : out std_ulogic_vector (127 downto 0);
    p_Consumer_ostream_eos : out std_ulogic;
    p_Consumer_ostream_rdy : out std_ulogic;
    p_sw_proc_output_stream_en : in std_ulogic;
    p_sw_proc_output_stream_data : out std_ulogic_vector (127 downto 0);
    p_sw_proc_output_stream_eos : out std_ulogic;
    p_sw_proc_output_stream_rdy : out std_ulogic;
    p_sw_proc_y_stream_en : in std_ulogic;
    p_sw_proc_y_stream_eos : in std_ulogic;
    p_sw_proc_y_stream_data : in std_ulogic_vector (127 downto 0);
    p_sw_proc_y_stream_rdy : out std_ulogic;
    p_sw_proc_u_stream_en : in std_ulogic;
    p_sw_proc_u_stream_eos : in std_ulogic;
    p_sw_proc_u_stream_data : in std_ulogic_vector (127 downto 0);
    p_sw_proc_u_stream_rdy : out std_ulogic;
    p_sw_proc_v_stream_en : in std_ulogic;
    p_sw_proc_v_stream_eos : in std_ulogic;
    p_sw_proc_v_stream_data : in std_ulogic_vector (127 downto 0);
    p_sw_proc_v_stream_rdy : out std_ulogic
    );
  end component;

  signal p_Producer_istream_rdy : std_ulogic;
  signal p_Producer_istream_en : std_ulogic;
  signal p_Producer_istream_eos : std_ulogic;
  signal p_Producer_istream_idata : std_ulogic_vector (127 downto 0);
  signal p_Consumer_ostream_rdy : std_ulogic;
  signal p_Consumer_ostream_en : std_ulogic;
  signal p_Consumer_ostream_eos : std_ulogic;
  signal p_Consumer_ostream_idata : std_ulogic_vector (127 downto 0);
  signal p_sw_proc_output_stream_rdy : std_ulogic;
  signal p_sw_proc_output_stream_en : std_ulogic;
  signal p_sw_proc_output_stream_eos : std_ulogic;
  signal p_sw_proc_output_stream_idata : std_ulogic_vector (127 downto 0);
  signal p_sw_proc_y_stream_rdy : std_ulogic;
  signal p_sw_proc_y_stream_en : std_ulogic;
  signal p_sw_proc_y_stream_eos : std_ulogic;
  signal p_sw_proc_y_stream_idata : std_ulogic_vector (127 downto 0);
  signal p_sw_proc_u_stream_rdy : std_ulogic;
  signal p_sw_proc_u_stream_en : std_ulogic;
  signal p_sw_proc_u_stream_eos : std_ulogic;
  signal p_sw_proc_u_stream_idata : std_ulogic_vector (127 downto 0);
  signal p_sw_proc_v_stream_rdy : std_ulogic;
  signal p_sw_proc_v_stream_en : std_ulogic;
  signal p_sw_proc_v_stream_eos : std_ulogic;
  signal p_sw_proc_v_stream_idata : std_ulogic_vector (127 downto 0);

  signal p_Producer_istream_storedata : std_logic_vector(0 to 127);
  signal p_Producer_istream_done : std_logic;
  signal p_Producer_istream_sleepnotready : std_logic;
  signal p_Producer_istream_resultvalid : std_logic;
  signal p_Producer_istream_cs : std_logic;
  signal p_Producer_istream_instrvalid : std_logic;
  signal p_Consumer_ostream_storedata : std_logic_vector(0 to 127);
  signal p_Consumer_ostream_done : std_logic;
  signal p_Consumer_ostream_sleepnotready : std_logic;
  signal p_Consumer_ostream_resultvalid : std_logic;
  signal p_Consumer_ostream_cs : std_logic;
  signal p_Consumer_ostream_instrvalid : std_logic;
  signal p_sw_proc_output_stream_storedata : std_logic_vector(0 to 127);
  signal p_sw_proc_output_stream_done : std_logic;
  signal p_sw_proc_output_stream_sleepnotready : std_logic;
  signal p_sw_proc_output_stream_resultvalid : std_logic;
  signal p_sw_proc_output_stream_cs : std_logic;
  signal p_sw_proc_output_stream_instrvalid : std_logic;
  signal p_sw_proc_y_stream_storedata : std_logic_vector(0 to 127);
  signal p_sw_proc_y_stream_done : std_logic;
  signal p_sw_proc_y_stream_sleepnotready : std_logic;
  signal p_sw_proc_y_stream_resultvalid : std_logic;
  signal p_sw_proc_y_stream_cs : std_logic;
  signal p_sw_proc_y_stream_instrvalid : std_logic;
  signal p_sw_proc_u_stream_storedata : std_logic_vector(0 to 127);
  signal p_sw_proc_u_stream_done : std_logic;
  signal p_sw_proc_u_stream_sleepnotready : std_logic;
  signal p_sw_proc_u_stream_resultvalid : std_logic;
  signal p_sw_proc_u_stream_cs : std_logic;
  signal p_sw_proc_u_stream_instrvalid : std_logic;
  signal p_sw_proc_v_stream_storedata : std_logic_vector(0 to 127);
  signal p_sw_proc_v_stream_done : std_logic;
  signal p_sw_proc_v_stream_sleepnotready : std_logic;
  signal p_sw_proc_v_stream_resultvalid : std_logic;
  signal p_sw_proc_v_stream_cs : std_logic;
  signal p_sw_proc_v_stream_instrvalid : std_logic;

  -- apu440 signals
  signal apusel : std_logic_vector(0 to 2);
  signal nonautonomousWildcardDecodeValid : std_logic;
  signal autonomousWildcardDecodeValid : std_logic;
  signal wildcardDecode : std_logic_vector(0 to 3);
  signal fcm_transaction_id : unsigned (31 downto 0);
  -- end apu440 signals
begin

  udidebug(0) <= apu_clk;
  udidebug(1) <= fcmapuresultvalid;
  udidebug(2) <= fcmapudone;
  udidebug(3) <= fcmapusleepnotready;
  udidebug(4) <= fcmapuexception;
  udidebug(5) <= fcmapuconfirminstr;

  udidebug(37 downto 6) <= apufcminstruction;
  udidebug(38) <= apufcmdecudivalid;
  udidebug(39) <= apufcmflush;
  udidebug(40) <= apufcmwritebackok;
  udidebug(41) <= apufcminstrvalid;
  udidebug(42) <= apufcmoperandvalid;
  udidebug(43) <= apufcmdecnonauton;
  udidebug(44) <= apufcmdecstore;
  udidebug(45) <= apufcmdecload;
  udidebug(46) <= apufcmloaddvalid;
  udidebug(49 downto 47) <= apufcmdecldstxfersize;
  udidebug(53 downto 50) <= apufcmdecudi;
  udidebug(85 downto 54) <= apufcmradata;
  udidebug(117 downto 86) <= fcmapuresult;
  udidebug(121 downto 118) <= apufcmloadbyteaddr;
  udidebug(153 downto 122) <= std_logic_vector(fcm_transaction_id);

  process (reset, apu_clk)
  begin
    if (reset = '1') then
	fcm_transaction_id <= X"00000000";
    elsif (apu_clk'event and apu_clk='1') then
        if (apufcminstrvalid = '1' and fcmapudone = '1') then
	  fcm_transaction_id <= fcm_transaction_id + X"00000001";
        end if;
    end if;
  end process;


  apusel <= apufcminstruction(6 to 8);

  nonautonomousWildcardDecodeValid <= '1' when apufcmdecudivalid = '1' and apufcmdecudi = "0000" else '0';
  autonomousWildcardDecodeValid <= '1' when apufcmdecudivalid = '1' and apufcmdecudi = "0001" else '0';
  wildcardDecode <= apufcminstruction(22 to 25);

  p_Producer_istream_cs <= '1' when apusel = "000" else '0';
  p_Producer_istream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_Producer_istream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_Producer_istream_if: apu_to_stream
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_Producer_istream_cs,
      apufcminstruction,
      p_Producer_istream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_Producer_istream_storedata,
      p_Producer_istream_done,
      p_Producer_istream_sleepnotready,
      p_Producer_istream_resultvalid,
      p_Producer_istream_en,
      p_Producer_istream_rdy,
      p_Producer_istream_eos,
      p_Producer_istream_idata
    );

  p_Consumer_ostream_cs <= '1' when apusel = "001" else '0';
  p_Consumer_ostream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_Consumer_ostream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_Consumer_ostream_if: stream_to_apu
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_Consumer_ostream_cs,
      apufcminstruction,
      p_Consumer_ostream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_Consumer_ostream_storedata,
      p_Consumer_ostream_done,
      p_Consumer_ostream_sleepnotready,
      p_Consumer_ostream_resultvalid,
      p_Consumer_ostream_en,
      p_Consumer_ostream_rdy,
      p_Consumer_ostream_eos,
      p_Consumer_ostream_idata
    );

  p_sw_proc_output_stream_cs <= '1' when apusel = "010" else '0';
  p_sw_proc_output_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_sw_proc_output_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_sw_proc_output_stream_if: stream_to_apu
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_sw_proc_output_stream_cs,
      apufcminstruction,
      p_sw_proc_output_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_sw_proc_output_stream_storedata,
      p_sw_proc_output_stream_done,
      p_sw_proc_output_stream_sleepnotready,
      p_sw_proc_output_stream_resultvalid,
      p_sw_proc_output_stream_en,
      p_sw_proc_output_stream_rdy,
      p_sw_proc_output_stream_eos,
      p_sw_proc_output_stream_idata
    );

  p_sw_proc_y_stream_cs <= '1' when apusel = "011" else '0';
  p_sw_proc_y_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_sw_proc_y_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_sw_proc_y_stream_if: apu_to_stream
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_sw_proc_y_stream_cs,
      apufcminstruction,
      p_sw_proc_y_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_sw_proc_y_stream_storedata,
      p_sw_proc_y_stream_done,
      p_sw_proc_y_stream_sleepnotready,
      p_sw_proc_y_stream_resultvalid,
      p_sw_proc_y_stream_en,
      p_sw_proc_y_stream_rdy,
      p_sw_proc_y_stream_eos,
      p_sw_proc_y_stream_idata
    );

  p_sw_proc_u_stream_cs <= '1' when apusel = "100" else '0';
  p_sw_proc_u_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_sw_proc_u_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_sw_proc_u_stream_if: apu_to_stream
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_sw_proc_u_stream_cs,
      apufcminstruction,
      p_sw_proc_u_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_sw_proc_u_stream_storedata,
      p_sw_proc_u_stream_done,
      p_sw_proc_u_stream_sleepnotready,
      p_sw_proc_u_stream_resultvalid,
      p_sw_proc_u_stream_en,
      p_sw_proc_u_stream_rdy,
      p_sw_proc_u_stream_eos,
      p_sw_proc_u_stream_idata
    );

  p_sw_proc_v_stream_cs <= '1' when apusel = "101" else '0';
  p_sw_proc_v_stream_instrvalid <= '1' when (apufcminstrvalid = '1') and (p_sw_proc_v_stream_cs = '1') and apufcmdecfpuop = '0' else '0';
  p_sw_proc_v_stream_if: apu_to_stream
    generic map (
      datawidth => 128
    )
    port map (
      reset, apu_clk,
      p_sw_proc_v_stream_cs,
      apufcminstruction,
      p_sw_proc_v_stream_instrvalid,
      apufcmradata,
      apufcmrbdata,
      apufcmoperandvalid,
      apufcmflush,
      apufcmwritebackok,
      apufcmloaddata,
      apufcmloaddvalid,
      apufcmloadbyteaddr,
      apufcmendian,
      apufcmdecudi,
      apufcmdecudivalid,
      p_sw_proc_v_stream_storedata,
      p_sw_proc_v_stream_done,
      p_sw_proc_v_stream_sleepnotready,
      p_sw_proc_v_stream_resultvalid,
      p_sw_proc_v_stream_en,
      p_sw_proc_v_stream_rdy,
      p_sw_proc_v_stream_eos,
      p_sw_proc_v_stream_idata
    );

  fcmapustoredata <= p_Producer_istream_storedata or p_Consumer_ostream_storedata or p_sw_proc_output_stream_storedata or p_sw_proc_y_stream_storedata or p_sw_proc_u_stream_storedata or p_sw_proc_v_stream_storedata;
  fcmapudone <= p_Producer_istream_done or p_Consumer_ostream_done or p_sw_proc_output_stream_done or p_sw_proc_y_stream_done or p_sw_proc_u_stream_done or p_sw_proc_v_stream_done;
  fcmapusleepnotready <= p_Producer_istream_sleepnotready or p_Consumer_ostream_sleepnotready or p_sw_proc_output_stream_sleepnotready or p_sw_proc_y_stream_sleepnotready or p_sw_proc_u_stream_sleepnotready or p_sw_proc_v_stream_sleepnotready;
  fcmapuresultvalid <= p_Producer_istream_resultvalid or p_Consumer_ostream_resultvalid or p_sw_proc_output_stream_resultvalid or p_sw_proc_y_stream_resultvalid or p_sw_proc_u_stream_resultvalid or p_sw_proc_v_stream_resultvalid;
  fcmapuresult <= X"00000000";
  fcmapucr <= "0000";

  -- unused signals
  fcmapuexception <= '0';
  fcmapuconfirminstr <= '1';
  fcmapufpscrfex <= '0';

  idct_colorconv_arch_0: idct_colorconv_arch
    port map (
      reset,
      apu_clk,
      apu_clk,
      p_Producer_istream_en,
      p_Producer_istream_eos,
      p_Producer_istream_idata,
      p_Producer_istream_rdy,
      p_Consumer_ostream_en,
      p_Consumer_ostream_idata,
      p_Consumer_ostream_eos,
      p_Consumer_ostream_rdy,
      p_sw_proc_output_stream_en,
      p_sw_proc_output_stream_idata,
      p_sw_proc_output_stream_eos,
      p_sw_proc_output_stream_rdy,
      p_sw_proc_y_stream_en,
      p_sw_proc_y_stream_eos,
      p_sw_proc_y_stream_idata,
      p_sw_proc_y_stream_rdy,
      p_sw_proc_u_stream_en,
      p_sw_proc_u_stream_eos,
      p_sw_proc_u_stream_idata,
      p_sw_proc_u_stream_rdy,
      p_sw_proc_v_stream_en,
      p_sw_proc_v_stream_eos,
      p_sw_proc_v_stream_idata,
      p_sw_proc_v_stream_rdy);

end;

