//////////////////////////////////////////////////////////////////
// idct_hw.c: includes the hardware process and configuration
// function.
//
// See additional comments in idct.h.
//

#include <stdio.h>
#include "co.h"
#include "idct.h"
 
extern void Producer(co_stream istream);
extern void Consumer(co_stream ostream);

#ifdef color
extern void Color_Convert_Temp(
co_stream output_stream, 
co_stream y_stream, 
co_stream u_stream, 
co_stream v_stream);
#endif

#define W1 2841 /* 2048*sqrt(2)*cos(1*pi/16) */
#define W2 2676 /* 2048*sqrt(2)*cos(2*pi/16) */
#define W3 2408 /* 2048*sqrt(2)*cos(3*pi/16) */
#define W5 1609 /* 2048*sqrt(2)*cos(5*pi/16) */
#define W6 1108 /* 2048*sqrt(2)*cos(6*pi/16) */
#define W7 565  /* 2048*sqrt(2)*cos(7*pi/16) */

#ifdef IMPULSE_C_SYNTHESIS
#define GETWORD7(wide) ((short)wide)
#define GETWORD6(wide) ((short)(wide>>16))
#define GETWORD5(wide) ((short)(wide>>32))
#define GETWORD4(wide) ((short)(wide>>48))
#define GETWORD3(wide) ((short)(wide>>64))
#define GETWORD2(wide) ((short)(wide>>80))
#define GETWORD1(wide) ((short)(wide>>96))
#define GETWORD0(wide) ((short)(wide>>112))
#define PACK(dst,a,b,c,d,e,f,g,h)\
	dst = (co_uint128)(co_uint24)h|\
		((co_uint128)(co_uint24)g)<<16|\
		((co_uint128)(co_uint24)f)<<32|\
		((co_uint128)(co_uint24)e)<<48|\
		((co_uint128)(co_uint24)d)<<64|\
		((co_uint128)(co_uint24)c)<<80|\
		((co_uint128)(co_uint24)b)<<96|\
		((co_uint128)(co_uint24)a)<<112
#define PUSHWORD(wide,val) wide=((co_uint16)(val))|(wide<<16)
#else
#define GETWORD6(wide) ((short)wide[3])
#define GETWORD7(wide) ((short)(wide[3]>>16))
#define GETWORD4(wide) ((short)wide[2])
#define GETWORD5(wide) ((short)(wide[2]>>16))
#define GETWORD2(wide) ((short)wide[1])
#define GETWORD3(wide) ((short)(wide[1]>>16))
#define GETWORD0(wide) ((short)wide[0])
#define GETWORD1(wide) ((short)(wide[0]>>16))
#define PUSHWORD(wide,val) \
  wide[0]=(wide[0]>>16)|(wide[1]<<16);\
  wide[1]=(wide[1]>>16)|(wide[2]<<16);\
  wide[2]=(wide[2]>>16)|(wide[3]<<16);\
  wide[3]=(wide[3]>>16)|((val)<<16)
#define PACK(wide,a,b,c,d,e,f,g,h) \
  wide[0]=(b<<16)|(a&0xffff);\
  wide[1]=(d<<16)|(c&0xffff);\
  wide[2]=(f<<16)|(e&0xffff);\
  wide[3]=(h<<16)|(g&0xffff)
#endif

#define mul(x,y) ((co_int16)(x)*(co_int16)(y))
#define mul32(x,y) ((x)*(y))

co_int32 mul16x32(co_int16 a, co_int32 b)

{
#pragma CO implementation mul_16x32_32 logic
  co_int18 low;
  co_int18 high;
  co_int32 res;

  low=b&0x1ffff;
  high=b>>17;
  
  res=low*a;
  res+=(a*high)<<17;
  return res;
}

void idctrow(co_stream shifts, co_stream rows, co_stream rowout)

{
  co_uint1 shift;
  co_uint8 i;
  co_uint128 row;
  co_int24 x0, x1, x2, x3, x4, x5, x6, x7, x8;
  co_int32 acc0,acc1,acc2,acc3,acc4,acc5,acc6,acc7;

  co_stream_open(shifts, O_RDONLY, UINT_TYPE(1));
  co_stream_open(rows, O_RDONLY, UINT_TYPE(128));
  co_stream_open(rowout, O_WRONLY, UINT_TYPE(128));
  
  do {
#pragma co pipeline
//#pragma co set stagedelay 64
		// pipeline latency must not be greater than 8 to avoid pipeline stalling 
		//  that will cause inter-process gridlock 
	  co_stream_read(shifts,&shift,sizeof(co_uint1));
	  co_stream_read(rows,&row,sizeof(co_uint128));
	  x1 = GETWORD4(row);
	  x2 = GETWORD6(row);
	  x3 = GETWORD2(row);
	  x4 = GETWORD1(row);
	  x5 = GETWORD7(row);
	  x6 = GETWORD5(row);
	  x7 = GETWORD3(row);
	  x0 = GETWORD0(row);

  // first stage 
  // second stage 
  // third stage 
    acc0 = x4+x5;
    acc2 = x6+x7;
    acc4 = x3+x2;
co_par_break();
    acc0 = acc1 = mul16x32(W7,acc0);
    acc2 = acc3 = mul16x32(W3,acc2);
    acc4 = acc5 = mul16x32(W6,acc4);
    acc6 = ((co_int32)(x0+x1)<<11)+128;
    acc7 = ((co_int32)(x0-x1)<<11)+128;
co_par_break();
	acc0 = acc0 + mul(W1-W7,x4);
    acc1 = acc1 - mul(W1+W7,x5);
	acc2 = acc2 - mul(W3-W5,x6);
	acc3 = acc3 - mul(W3+W5,x7);
  	acc4 = acc4 - mul(W2+W6,x2);
	acc5 = acc5 + mul(W2-W6,x3);
co_par_break();
    if (shift) {
	  acc0>>=3;
	  acc1>>=3;
	  acc2>>=3;
	  acc3>>=3;
	  acc4>>=3;
	  acc5>>=3;
	  acc6>>=3;
	  acc7>>=3;
	}
co_par_break();
    x1 = acc0 + acc2;
    x6 = acc1 + acc3;
    x7 = acc6 + acc5;
    x8 = acc6 - acc5;
    x3 = acc7 + acc4;
    x0 = acc7 - acc4;
    acc0 = acc0 - acc2;
    acc1 = acc1 - acc3;
	acc2 = acc0 + acc1;
	acc3 = acc0 - acc1;
co_par_break();
#if 0
#if 0
    x2 = (mul16x32(181,acc2)+128)>>8;
    x4 = (mul16x32(181,acc3)+128)>>8;
#else
	// separate add from multiply for latency+1 but faster clock
	{
		co_int25 x2_tmp, x4_tmp;
    x2_tmp = mul16x32(181,acc2)>>7;
    x4_tmp = mul16x32(181,acc3)>>7;
	 //co_par_break();
	 x2 = (x2_tmp + 1) >> 1;
	 x4 = (x4_tmp + 1) >> 1;
	}
#endif
   co_par_break();
	PACK(row,(x7+x1)>>8,(x3+x2)>>8,(x0+x4)>>8,(x8+x6)>>8,(x8-x6)>>8,(x0-x4)>>8,(x3-x2)>>8,(x7-x1)>>8);
	co_stream_write(rowout,&row,sizeof(co_uint128));
#else
	// separate add from multiply for latency+1 but faster clock
	{
		co_int25 x2_mul, x4_mul;
		co_int24 x2_rnd, x4_rnd;
    x2_mul = mul16x32(181,acc2)>>7;
    x4_mul = mul16x32(181,acc3)>>7;
	 co_par_break();
	 x2_rnd = (x2_mul + 1) >> 1;
	 x4_rnd = (x4_mul + 1) >> 1;
	PACK(row,(x7+x1)>>8,
				(x3+x2_rnd)>>8,
				(x0+x4_rnd)>>8,
				(x8+x6)>>8,
				(x8-x6)>>8,
				(x0-x4_rnd)>>8,
				(x3-x2_rnd)>>8,
				(x7-x1)>>8
			);
	}
	co_stream_write(rowout,&row,sizeof(co_uint128));
	
#endif
  } while (1);

  co_stream_close(shifts);
  co_stream_close(rows);
  co_stream_close(rowout);
}

#define COPY(var,dst,src)\
  var ## dst ## _0=var ## src ## _0;\
  var ## dst ## _1=var ## src ## _1;\
  var ## dst ## _2=var ## src ## _2;\
  var ## dst ## _3=var ## src ## _3;\
  var ## dst ## _4=var ## src ## _4;\
  var ## dst ## _5=var ## src ## _5;\
  var ## dst ## _6=var ## src ## _6;\
  var ## dst ## _7=var ## src ## _7

#define PUSHREG(var,row,val)\
  var ## row ## _0=var ## row ## _1; var ## row ## _1=var ## row ## _2; var ## row ## _2=var ## row ## _3; var ## row ## _3=var ## row ## _4;\
  var ## row ## _4=var ## row ## _5; var ## row ## _5=var ## row ## _6; var ## row ## _6=var ## row ## _7; var ## row ## _7=val

void ctrl(co_stream rows, co_stream shifts, co_stream bufin, co_stream bufout, co_stream rowout)

{
  co_uint1 shift;
  co_uint8 i;
  co_int24 x0, x1, x2, x3, x4, x5, x6, x7, x8;
  co_int32 acc0,acc1,acc2,acc3,acc4,acc5,acc6,acc7;
  co_uint16 regs0_0,regs0_1,regs0_2,regs0_3,regs0_4,regs0_5,regs0_6,regs0_7;
  co_uint16 regs1_0,regs1_1,regs1_2,regs1_3,regs1_4,regs1_5,regs1_6,regs1_7;
  co_uint16 regs2_0,regs2_1,regs2_2,regs2_3,regs2_4,regs2_5,regs2_6,regs2_7;
  co_uint16 regs3_0,regs3_1,regs3_2,regs3_3,regs3_4,regs3_5,regs3_6,regs3_7;
  co_uint16 regs4_0,regs4_1,regs4_2,regs4_3,regs4_4,regs4_5,regs4_6,regs4_7;
  co_uint16 regs5_0,regs5_1,regs5_2,regs5_3,regs5_4,regs5_5,regs5_6,regs5_7;
  co_uint16 regs6_0,regs6_1,regs6_2,regs6_3,regs6_4,regs6_5,regs6_6,regs6_7;
  co_uint16 regs7_0,regs7_1,regs7_2,regs7_3,regs7_4,regs7_5,regs7_6,regs7_7;
  co_uint128 buf1,buf2;

  co_stream_open(rows, O_RDONLY, UINT_TYPE(128));
  co_stream_open(shifts, O_WRONLY, UINT_TYPE(1));
  co_stream_open(bufin, O_WRONLY, UINT_TYPE(128));
  co_stream_open(bufout, O_RDONLY, UINT_TYPE(128));
  co_stream_open(rowout, O_WRONLY, UINT_TYPE(128));

  for (i=0; i<8; i++) {
	  co_stream_write(shifts,&shift,sizeof(co_uint1));
	  co_stream_write(bufin,&buf1,sizeof(co_uint128));
  }
	
  do {
	i=0;
    do {
	  //#pragma co pipeline
	  shift=0;
	  co_stream_read(rows,&buf1,sizeof(co_uint128));
	  co_stream_read(bufout,&buf2,sizeof(co_uint128));
	  co_stream_write(shifts,&shift,sizeof(co_uint1));
	  co_stream_write(bufin,&buf1,sizeof(co_uint128));
	  PUSHREG(regs,0,GETWORD0(buf2)>>6);
      PUSHREG(regs,1,GETWORD1(buf2)>>6);
      PUSHREG(regs,2,GETWORD2(buf2)>>6);
      PUSHREG(regs,3,GETWORD3(buf2)>>6);
      PUSHREG(regs,4,GETWORD4(buf2)>>6);
      PUSHREG(regs,5,GETWORD5(buf2)>>6);
      PUSHREG(regs,6,GETWORD6(buf2)>>6);
      PUSHREG(regs,7,GETWORD7(buf2)>>6);
	  i++;
	} while (i<8);
    do {
	  //#pragma co pipeline
	  shift=0;
	  PACK(buf2,regs0_0,regs0_1,regs0_2,regs0_3,regs0_4,regs0_5,regs0_6,regs0_7);
	  co_stream_write(rowout,&buf2,sizeof(co_uint128));
	  co_stream_read(bufout,&buf1,sizeof(co_uint128));
	  co_stream_write(shifts,&shift,sizeof(co_uint1));
	  co_stream_write(bufin,&buf2,sizeof(co_uint128));
	  COPY(regs,0,1);
	  COPY(regs,1,2);
	  COPY(regs,2,3);
	  COPY(regs,3,4);
	  COPY(regs,4,5);
	  COPY(regs,5,6);
	  COPY(regs,6,7);
	  regs7_0 = GETWORD0(buf1);
	  regs7_1 = GETWORD1(buf1);
	  regs7_2 = GETWORD2(buf1);
	  regs7_3 = GETWORD3(buf1);
	  regs7_4 = GETWORD4(buf1);
	  regs7_5 = GETWORD5(buf1);
	  regs7_6 = GETWORD6(buf1);
	  regs7_7 = GETWORD7(buf1);
	  i++;
    } while (i<16);
    do {
	  #pragma co pipeline
	  shift=1;
	  co_stream_read(bufout,&buf2,sizeof(co_uint128));
	  PACK(buf1,regs0_0,regs1_0,regs2_0,regs3_0,regs4_0,regs5_0,regs6_0,regs7_0);
	  co_stream_write(shifts,&shift,sizeof(co_uint1));
	  co_stream_write(bufin,&buf1,sizeof(co_uint128));
	  PUSHREG(regs,0,0);
      PUSHREG(regs,1,0);
      PUSHREG(regs,2,0);
      PUSHREG(regs,3,0);
      PUSHREG(regs,4,0);
      PUSHREG(regs,5,0);
      PUSHREG(regs,6,0);
      PUSHREG(regs,7,0);
	  i++;
	} while (i<24);
  } while (1);

  co_stream_close(rows);
  co_stream_close(shifts);
  co_stream_close(bufin);
  co_stream_close(bufout);
  co_stream_close(rowout);
}

#ifdef color

co_int24 yuvmul(co_int16 a, co_int16 b)

{
#pragma CO implementation mul_16x16_24 logic
  return a*b;
}

void Color_Convert(co_stream y_stream, co_stream u_stream, co_stream v_stream, co_stream output_stream) //co_stream u_stream, co_stream v_stream,
{
	co_uint128 y_16, u_16, v_16, output_16, i_16;
	co_int32 YY1,YY2,rp,gp1,gp2,bp;
	co_int16 UU,VV, R, G, B;
	co_uint32 output_4;
	co_int4 i;

    co_stream_open(output_stream, O_WRONLY, UINT_TYPE(128));
	co_stream_open(y_stream, O_RDONLY, UINT_TYPE(128));
	co_stream_open(u_stream, O_RDONLY, UINT_TYPE(128));
	co_stream_open(v_stream, O_RDONLY, UINT_TYPE(128));

///////////yuv/////////////////////
	i=-1;
	do {
#pragma co pipeline
		i++;
		if ((i&0x7)==0) {
			co_stream_read(y_stream,&y_16,sizeof(co_uint128));
		}
		if ((i&0xf)==0){
			co_stream_read(u_stream,&u_16,sizeof(co_uint128));
			co_stream_read(v_stream,&v_16,sizeof(co_uint128));
		}

// Does a logical shift right operation on 128-bit integer 'a' 
// of between 128 and 96 bit places ('n') and returns the 
// least significant byte of the result.
#ifdef IMPULSE_C_SYNTHESIS
#define SRL128to96_BYTE(a,n) ((co_uint8)((a)>>(n)))
#else
#define SRL128to96_BYTE(a,n) ((co_uint8)((a)[3]>>((n)-128+32)))
#endif

// Does a logical shift left operation on 128-bit integer 'a'
// of between 32 and 0 bit places ('n') and assigns the result
// back to 'a'.
#ifdef IMPULSE_C_SYNTHESIS
#define SLL32to0_ASSIGN128b(a,n) (a)=(a)<<(n)
#else
#define SLL32to0_ASSIGN128b(a,n) \
	(a)[3] = ((a)[3] << (n)) & (((co_uint32)(a)[2]) >> (32-(n)));\
	(a)[2] = ((a)[2] << (n)) & (((co_uint32)(a)[1]) >> (32-(n)));\
	(a)[1] = ((a)[1] << (n)) & (((co_uint32)(a)[0]) >> (32-(n)));\
	(a)[0] = ((a)[0] << (n))
#endif

		YY1 = ((co_int32)SRL128to96_BYTE(y_16,120)) << 8;
		YY2 = ((co_int32)SRL128to96_BYTE(y_16,112)) << 8;
		VV  = ((co_int16)SRL128to96_BYTE(v_16,120)) - 128;
		UU  = ((co_int16)SRL128to96_BYTE(u_16,120)) - 128;
		SLL32to0_ASSIGN128b(y_16,16);
		SLL32to0_ASSIGN128b(u_16,8);
		SLL32to0_ASSIGN128b(v_16,8);
		co_par_break();
		rp=yuvmul(351 , VV);
		gp1=yuvmul(179 , VV);
		gp2=yuvmul(87 , UU);
		bp=yuvmul(443 , UU);
		co_par_break();
		R  = (co_int16)((YY1 + rp) >> 8);
		G  = (co_int16)((YY1 - gp1 - gp2) >> 8);
		B  = (co_int16)((YY1 + bp) >> 8);
		if(R<0)R=0;
		if(G<0)G=0;
		if(B<0)B=0;
		if(R>255)R=255;
		if(G>255)G=255;
		if(B>255)B=255;
		output_4 = (   ((co_uint32)R<<16) | ((co_uint32)G<<8) | (co_uint32)B       );
#ifdef IMPULSE_C_SYNTHESIS
		output_16 = (co_uint128)output_4 | (output_16 << 32);
#else
		output_16[0] = output_4;
#endif
		R  = (co_int16)((YY2 + rp) >> 8);
		G  = (co_int16)((YY2 - gp1 - gp2) >> 8);
		B  = (co_int16)((YY2 + bp) >> 8);
		if(R<0)R=0;
		if(G<0)G=0;
		if(B<0)B=0;
		if(R>255)R=255;
		if(G>255)G=255;
		if(B>255)B=255;
		output_4 = (   ((co_uint32)R<<16) | ((co_uint32)G<<8) | (co_uint32)B       );
#ifdef IMPULSE_C_SYNTHESIS
		output_16 = (co_uint128)output_4 | (output_16 << 32);
#else
		output_16[0] = output_4;
#endif

		if ((i&0x1)==1) {
			co_stream_write(output_stream,&output_16,sizeof(co_uint128));
		}
	} while (1);

    co_stream_close(y_stream);
	co_stream_close(u_stream);
	co_stream_close(v_stream);
	co_stream_close(output_stream);

}
#endif

#ifndef DCT_TEST
//
// Impulse C configuration function
//
void config_idct(void *arg) 
{
//	int i;
	co_stream istream,shift,bufin,bufout,ostream;
#ifdef color	
	co_stream y_stream, u_stream, v_stream, idcti_stream, output_stream, idcto_stream;	//system streams
#endif
	co_process rowproc,ctrlproc;
	co_process producer_process;
	co_process consumer_process;
#ifdef color
	co_process sw_proc, hw_proc;	//system processes
#endif
	//idct
	istream = co_stream_create("istream", UINT_TYPE(128), IDCT_IN_STREAM_DEPTH);
	shift = co_stream_create("shift", UINT_TYPE(1), 4);
	bufout = co_stream_create("mid1", UINT_TYPE(128), 4);
	bufin = co_stream_create("mid2", UINT_TYPE(128), 4);
	ostream = co_stream_create("ostream", UINT_TYPE(128), IDCT_OUT_STREAM_DEPTH);
#ifdef USE_BRAM_STREAMS
	// can't use BRAM with dual-clock, there is no stream_bram_dc
	co_stream_config(istream,co_kind,"bram");
	co_stream_config(shift,co_kind,"bram");
	co_stream_config(bufout,co_kind,"bram");
	co_stream_config(bufin,co_kind,"bram");
	co_stream_config(ostream,co_kind,"bram");
#endif
#ifdef color
	//yuv
    output_stream = co_stream_create("output_stream", UINT_TYPE(128), YUV_OUT_STREAM_DEPTH);
    y_stream = co_stream_create("y_stream", UINT_TYPE(128), YUV_OUT_STREAM_DEPTH/2);
	u_stream = co_stream_create("u_stream", UINT_TYPE(128), YUV_OUT_STREAM_DEPTH/4);
	v_stream = co_stream_create("v_stream", UINT_TYPE(128), YUV_OUT_STREAM_DEPTH/4);
#ifdef USE_BRAM_STREAMS
	// can't use BRAM with dual-clock, there is no stream_bram_dc
	co_stream_config(output_stream,co_kind,"bram");
	co_stream_config(y_stream,co_kind,"bram");
	co_stream_config(u_stream,co_kind,"bram");
	co_stream_config(v_stream,co_kind,"bram");
#endif
#endif
	

	producer_process = co_process_create("Producer", (co_function)Producer,
                                         1,
                                         istream);

	rowproc = co_process_create("idct", (co_function)idctrow,
                                    3,
                                    shift,bufin,bufout);

	ctrlproc = co_process_create("controller", (co_function)ctrl,
                                    5,
                                    istream,shift,bufin,bufout,ostream);

	consumer_process = co_process_create("Consumer",(co_function)Consumer,
                                         1,
                                         ostream);
#ifdef color
	sw_proc = co_process_create("sw_proc", (co_function)Color_Convert_Temp, 
										4, 
										output_stream, y_stream, u_stream, v_stream);
	hw_proc = co_process_create("hw_proc", (co_function)Color_Convert, 
										4, 
										y_stream, u_stream, v_stream, output_stream);
#endif

	co_process_config(rowproc, co_loc, "PE0");  
	co_process_config(ctrlproc, co_loc, "PE0");
#ifdef color
	co_process_config(hw_proc, co_loc, "PE0");
#endif
}

co_architecture co_initialize(int param)
{
	return(co_architecture_create("idct_colorconv_arch","Generic",config_idct,(void *)param));
}
#endif
